@@ -1,6 +1,6 @@
 #!/usr/bin/python3 -i
 #
-# Copyright (c) 2021 The Khronos Group Inc.
+# Copyright (c) 2021-2022 The Khronos Group Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,27 +31,26 @@ class CommandValidationOutputGeneratorOptions(GeneratorOptions):
                  filename = None,
                  directory = '.',
                  genpath = None,
-                 apiname = None,
+                 apiname = 'vulkan',
                  profile = None,
                  versions = '.*',
                  emitversions = '.*',
-                 defaultExtensions = None,
+                 defaultExtensions = 'vulkan',
                  addExtensions = None,
                  removeExtensions = None,
                  emitExtensions = None,
                  emitSpirv = None,
                  sortProcedure = regSortFeatures,
-                 prefixText = "",
                  genFuncPointers = True,
                  protectFile = True,
-                 protectFeature = True,
-                 apicall = '',
-                 apientry = '',
-                 apientryp = '',
+                 protectFeature = False,
+                 apicall = 'VKAPI_ATTR ',
+                 apientry = 'VKAPI_CALL ',
+                 apientryp = 'VKAPI_PTR *',
                  indentFuncProto = True,
                  indentFuncPointer = False,
-                 alignFuncParam = 0,
-                 expandEnumerants = True,
+                 alignFuncParam = 48,
+                 expandEnumerants = False,
                  valid_usage_path = ''):
         GeneratorOptions.__init__(self,
                 conventions = conventions,
@@ -68,7 +67,6 @@ class CommandValidationOutputGeneratorOptions(GeneratorOptions):
                 emitExtensions = emitExtensions,
                 emitSpirv = emitSpirv,
                 sortProcedure = sortProcedure)
-        self.prefixText      = prefixText
         self.genFuncPointers = genFuncPointers
         self.protectFile     = protectFile
         self.protectFeature  = protectFeature
@@ -95,6 +93,7 @@ class CommandValidationOutputGenerator(OutputGenerator):
         self.alias_dict = dict()                          # Dict of cmd aliases
         self.header_file = False                          # Header file generation flag
         self.source_file = False                          # Source file generation flag
+        self.dynamic_states = []                          # VkDynamicState enum values
 
     #
     # Walk the JSON-derived dict and find all "vuid" key values
@@ -144,7 +143,7 @@ class CommandValidationOutputGenerator(OutputGenerator):
         copyright += '\n'
         copyright += '/***************************************************************************\n'
         copyright += ' *\n'
-        copyright += ' * Copyright (c) 2021 The Khronos Group Inc.\n'
+        copyright += ' * Copyright (c) 2021-2022 The Khronos Group Inc.\n'
         copyright += ' *\n'
         copyright += ' * Licensed under the Apache License, Version 2.0 (the "License");\n'
         copyright += ' * you may not use this file except in compliance with the License.\n'
@@ -169,6 +168,7 @@ class CommandValidationOutputGenerator(OutputGenerator):
         elif self.header_file:
             write('#pragma once', file=self.outFile)
             write('#include <array>', file=self.outFile)
+            write('#include <bitset>', file=self.outFile)
 
     #
     # Write generated file content to output file
@@ -176,12 +176,14 @@ class CommandValidationOutputGenerator(OutputGenerator):
         if self.header_file:
             write(self.commandTypeEnum(), file=self.outFile)
             write(self.commandNameList(), file=self.outFile)
+            write(self.dynamicTypeEnum(), file=self.outFile)
         elif self.source_file:
             write(self.commandRecordingList(), file=self.outFile)
             write(self.commandQueueTypeList(), file=self.outFile)
             write(self.commandRenderPassList(), file=self.outFile)
             write(self.commandBufferLevelList(), file=self.outFile)
             write(self.validateFunction(), file=self.outFile)
+            write(self.dynamicFunction(), file=self.outFile)
         # Finish processing in superclass
         OutputGenerator.endFile(self)
 
@@ -205,11 +207,16 @@ class CommandValidationOutputGenerator(OutputGenerator):
         params = cmdinfo.elem.findall('param')
         info = self.getTypeNameTuple(params[0])
         if name.startswith('vkCmd') and info[0] == 'VkCommandBuffer':
-            if alias is None:
-                self.commands[name] = cmdinfo
-            else:
+            self.commands[name] = cmdinfo
+            if alias is not None:
                 self.alias_dict[name] = alias
-
+    #
+    # List the enum for the commands
+    def genGroup(self, groupinfo, name, alias):
+        if (name == 'VkDynamicState'):
+            for elem in groupinfo.elem.findall('enum'):
+                if elem.get('alias') is None:
+                    self.dynamic_states.append(elem.get('name'))
     #
     # List the enum for the commands
     def commandTypeEnum(self):
@@ -240,6 +247,28 @@ static const std::array<const char *, CMD_RANGE_SIZE> kGeneratedCommandNameList
         output += '}};'
         return output
 
+    #
+    # List the enum for the dynamic command buffer status flags
+    def dynamicTypeEnum(self):
+        output = '''
+typedef enum CB_DYNAMIC_STATUS {\n'''
+        counter = 1
+        for name in self.dynamic_states:
+            state_name = name[17:] # VK_DYNAMIC_STATE_LINE_WIDTH -> LINE_WIDTH
+            output += '    CB_DYNAMIC_{}_SET = {},\n'.format(state_name, str(counter))
+            counter += 1
+
+        output += '    CB_DYNAMIC_STATUS_NUM = ' + str(counter)
+        output += '''
+} CB_DYNAMIC_STATUS;
+
+typedef std::bitset<CB_DYNAMIC_STATUS_NUM> CBDynamicFlags;
+std::string DynamicStateString(CBDynamicFlags const &dynamic_state);
+struct VkPipelineDynamicStateCreateInfo;
+CBDynamicFlags MakeStaticStateMask(VkPipelineDynamicStateCreateInfo const *info);
+'''
+        return output
+
     #
     # For each CMD_TYPE give a string name add a *-recording VUID
     # Each vkCmd* will have one
@@ -248,6 +277,8 @@ static const std::array<const char *, CMD_RANGE_SIZE> kGeneratedCommandNameList
 static const std::array<const char *, CMD_RANGE_SIZE> kGeneratedMustBeRecordingList = {{
     kVUIDUndefined,\n'''
         for name, cmdinfo in sorted(self.commands.items()):
+            if name in self.alias_dict:
+                name = self.alias_dict[name]
             vuid = 'VUID-' + name + '-commandBuffer-recording'
             if vuid not in self.valid_vuids:
                 print("Warning: Could not find {} in validusage.json".format(vuid))
@@ -268,6 +299,8 @@ struct CommandSupportedQueueType {
 static const std::array<CommandSupportedQueueType, CMD_RANGE_SIZE> kGeneratedQueueTypeList = {{
     {VK_QUEUE_FLAG_BITS_MAX_ENUM, kVUIDUndefined},\n'''
         for name, cmdinfo in sorted(self.commands.items()):
+            if name in self.alias_dict:
+                name = self.alias_dict[name]
             flags = []
             queues = cmdinfo.elem.attrib.get('queues').split(',')
             for queue in queues:
@@ -285,8 +318,10 @@ static const std::array<CommandSupportedQueueType, CMD_RANGE_SIZE> kGeneratedQue
                     flags.append("VK_QUEUE_VIDEO_DECODE_BIT_KHR")
                 elif queue == 'encode':
                     flags.append("VK_QUEUE_VIDEO_ENCODE_BIT_KHR")
+                elif queue == 'opticalflow':
+                    flags.append("VK_QUEUE_OPTICAL_FLOW_BIT_NV")
                 else:
-                    print("A new queue type %s was added to VkQueueFlagBits and need to update generation code", queue)
+                    print(f'A new queue type {queue} was added to VkQueueFlagBits and need to update generation code')
                     sys.exit(1)
             vuid = 'VUID-' + name + '-commandBuffer-cmdpool'
             if vuid not in self.valid_vuids:
@@ -312,6 +347,8 @@ struct CommandSupportedRenderPass {
 static const std::array<CommandSupportedRenderPass, CMD_RANGE_SIZE> kGeneratedRenderPassList = {{
     {CMD_RENDER_PASS_BOTH, kVUIDUndefined}, // CMD_NONE\n'''
         for name, cmdinfo in sorted(self.commands.items()):
+            if name in self.alias_dict:
+                name = self.alias_dict[name]
             render_pass_type = ''
             render_pass = cmdinfo.elem.attrib.get('renderpass')
             if render_pass == 'inside':
@@ -341,6 +378,8 @@ static const std::array<CommandSupportedRenderPass, CMD_RANGE_SIZE> kGeneratedRe
 static const std::array<const char *, CMD_RANGE_SIZE> kGeneratedBufferLevelList = {{
     kVUIDUndefined, // CMD_NONE\n'''
         for name, cmdinfo in sorted(self.commands.items()):
+            if name in self.alias_dict:
+                name = self.alias_dict[name]
             buffer_level = cmdinfo.elem.attrib.get('cmdbufferlevel')
             # Currently there is only "primary" or "primary,secondary" in XML
             # Hard to predict what might change, so will error out instead if assumption breaks
@@ -363,8 +402,9 @@ static const std::array<const char *, CMD_RANGE_SIZE> kGeneratedBufferLevelList
     def validateFunction(self):
         output = '''
 // Used to handle all the implicit VUs that are autogenerated from the registry
-bool CoreChecks::ValidateCmd(const CMD_BUFFER_STATE *cb_state, const CMD_TYPE cmd, const char *caller_name) const {
+bool CoreChecks::ValidateCmd(const CMD_BUFFER_STATE *cb_state, const CMD_TYPE cmd) const {
     bool skip = false;
+    const char *caller_name = CommandTypeString(cmd);
 
     // Validate the given command being added to the specified cmd buffer,
     // flagging errors if CB is not in the recording state or if there's an issue with the Cmd ordering
@@ -406,3 +446,59 @@ bool CoreChecks::ValidateCmd(const CMD_BUFFER_STATE *cb_state, const CMD_TYPE cm
     return skip;
 }'''
         return output
+
+    #
+    # List the enum for the dynamic command buffer status flags
+    def dynamicFunction(self):
+        output = '''
+static VkDynamicState ConvertToDynamicState(CB_DYNAMIC_STATUS flag) {
+    switch (flag) {\n'''
+        for name in self.dynamic_states:
+            state_name = name[17:] # VK_DYNAMIC_STATE_LINE_WIDTH -> LINE_WIDTH
+            output += '        case CB_DYNAMIC_{}_SET:\n'.format(state_name)
+            output += '            return {};\n'.format(name)
+        output += '''        default:
+            return VK_DYNAMIC_STATE_MAX_ENUM;
+    }
+}
+'''
+        output += '''
+static CB_DYNAMIC_STATUS ConvertToCBDynamicStatus(VkDynamicState state) {
+    switch (state) {\n'''
+        for name in self.dynamic_states:
+            state_name = name[17:] # VK_DYNAMIC_STATE_LINE_WIDTH -> LINE_WIDTH
+            output += '        case {}:\n'.format(name)
+            output += '            return CB_DYNAMIC_{}_SET;\n'.format(state_name)
+        output += '''        default:
+            return CB_DYNAMIC_STATUS_NUM;
+    }
+}
+'''
+
+        output += '''
+std::string DynamicStateString(CBDynamicFlags const &dynamic_state) {
+    std::string ret;
+    for (int index = 0; index < CB_DYNAMIC_STATUS_NUM; ++index) {
+        CB_DYNAMIC_STATUS status = static_cast<CB_DYNAMIC_STATUS>(index);
+        if (dynamic_state[status]) {
+            if (!ret.empty()) ret.append("|");
+            ret.append(string_VkDynamicState(ConvertToDynamicState(status)));
+        }
+    }
+    if (ret.empty()) ret.append(string_VkDynamicState(ConvertToDynamicState(CB_DYNAMIC_STATUS_NUM)));
+    return ret;
+}
+
+CBDynamicFlags MakeStaticStateMask(VkPipelineDynamicStateCreateInfo const *info) {
+    // initially assume everything is static state
+    CBDynamicFlags flags(~CBDynamicFlags(0));
+
+    if (info) {
+        for (uint32_t i = 0; i < info->dynamicStateCount; i++) {
+            flags.reset(ConvertToCBDynamicStatus(info->pDynamicStates[i]));
+        }
+    }
+    return flags;
+}
+'''
+        return output
\ No newline at end of file
