@@ -59,12 +59,18 @@ def makeGenOpts(args):
     # Extensions to emit (list of extensions)
     emitExtensions = args.emitExtensions
 
+    # Extensions to warn about, if enabled(list of extensions)
+    warnExtensions = args.warnExtensions
+
     # Features to include (list of features)
     features = args.feature
 
     # Spirv elements to emit (list of extensions and capabilities)
     emitSpirv = args.emitSpirv
 
+    # Format elements to emit
+    emitFormats = args.emitFormats
+
     # Whether to disable inclusion protect in headers
     protect = args.protect
 
@@ -76,7 +82,7 @@ def makeGenOpts(args):
 
     # Descriptive names for various regexp patterns used to select
     # versions and extensions
-    allFeatures     = allExtensions = allSpirv = '.*'
+    allFeatures     = allExtensions = allSpirv = allFormats = '.*'
     noFeatures      = noExtensions = noSpirv = None
 
     # Turn lists of names/patterns into matching regular expressions
@@ -85,35 +91,7 @@ def makeGenOpts(args):
     emitExtensionsPat    = makeREstring(emitExtensions, allExtensions)
     featuresPat          = makeREstring(features, allFeatures)
     emitSpirvPat         = makeREstring(emitSpirv, allSpirv)
-
-    # Copyright text prefixing all headers (list of strings).
-    prefixStrings = [
-        '/*',
-        '** Copyright (c) 2015-2019 The Khronos Group Inc.',
-        '**',
-        '** Licensed under the Apache License, Version 2.0 (the "License");',
-        '** you may not use this file except in compliance with the License.',
-        '** You may obtain a copy of the License at',
-        '**',
-        '**     http://www.apache.org/licenses/LICENSE-2.0',
-        '**',
-        '** Unless required by applicable law or agreed to in writing, software',
-        '** distributed under the License is distributed on an "AS IS" BASIS,',
-        '** WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.',
-        '** See the License for the specific language governing permissions and',
-        '** limitations under the License.',
-        '*/',
-        ''
-    ]
-
-    # Text specific to Vulkan headers
-    vkPrefixStrings = [
-        '/*',
-        '** This header is generated from the Khronos Vulkan XML API Registry.',
-        '**',
-        '*/',
-        ''
-    ]
+    emitFormatsPat       = makeREstring(emitFormats, allFormats)
 
     # Defaults for generating re-inclusion protection wrappers (or not)
     protectFeature = protect
@@ -129,23 +107,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'thread_safety.h',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
-            emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            protectFeature    = False,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants = False)
+            emitExtensions    = emitExtensionsPat)
         ]
 
     # Options for thread safety source code-generation
@@ -155,23 +121,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'thread_safety.cpp',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
-            emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            protectFeature    = False,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants = False)
+            emitExtensions    = emitExtensionsPat)
         ]
 
     # Options for stateless validation source file
@@ -181,22 +135,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'parameter_validation.cpp',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants  = False,
             valid_usage_path  = args.scripts)
           ]
 
@@ -207,22 +150,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'parameter_validation.h',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants  = False,
             valid_usage_path  = args.scripts)
           ]
 
@@ -233,23 +165,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'object_tracker.cpp',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            protectFeature    = False,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants  = False,
             valid_usage_path  = args.scripts)
         ]
 
@@ -260,23 +180,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'object_tracker.h',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            protectFeature    = False,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants  = False,
             valid_usage_path  = args.scripts)
         ]
 
@@ -287,22 +195,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'vk_dispatch_table_helper.h',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
-            emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants = False)
+            emitExtensions    = emitExtensionsPat)
         ]
 
     # lvt_file generator options for lvt_function_pointers.h
@@ -312,22 +209,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'lvt_function_pointers.h',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants = False,
             lvt_file_type  = 'function_pointer_header')
         ]
 
@@ -338,22 +224,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'lvt_function_pointers.cpp',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants = False,
             lvt_file_type  = 'function_pointer_source')
         ]
 
@@ -364,22 +239,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'vk_layer_dispatch_table.h',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
-            emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants = False)
+            emitExtensions    = emitExtensionsPat)
         ]
 
     # Helper file generator options for vk_enum_string_helper.h
@@ -389,22 +253,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'vk_enum_string_helper.h',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants  = False,
             helper_file_type  = 'enum_string_header')
         ]
 
@@ -415,22 +268,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'vk_safe_struct.h',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants  = False,
             helper_file_type  = 'safe_struct_header')
         ]
 
@@ -441,22 +283,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'vk_safe_struct.cpp',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants  = False,
             helper_file_type  = 'safe_struct_source')
         ]
 
@@ -467,22 +298,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'vk_object_types.h',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants  = False,
             helper_file_type  = 'object_types_header')
         ]
 
@@ -493,22 +313,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'vk_extension_helper.h',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants  = False,
             helper_file_type  = 'extension_helper_header')
         ]
 
@@ -519,23 +328,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'vk_typemap_helper.h',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            protectFeature    = False,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants  = False,
             helper_file_type  = 'typemap_helper_header')
         ]
 
@@ -546,21 +343,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'corechecks_optick_instrumentation.h',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants  = False,
             helper_file_type  = 'optick_instrumentation_header')
         ]
 
@@ -571,21 +358,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'corechecks_optick_instrumentation.cpp',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants  = False,
             helper_file_type  = 'optick_instrumentation_source')
         ]
 
@@ -597,23 +374,13 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'chassis.h',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            helper_file_type  = 'layer_chassis_header',
-            expandEnumerants = False)
+            warnExtensions    = warnExtensions,
+            helper_file_type  = 'layer_chassis_header')
         ]
 
     # Options for layer chassis source file
@@ -623,23 +390,13 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'chassis.cpp',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            helper_file_type  = 'layer_chassis_source',
-            expandEnumerants = False)
+            warnExtensions    = warnExtensions,
+            helper_file_type  = 'layer_chassis_source')
         ]
 
     # Layer chassis related generation structs
@@ -650,23 +407,12 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'chassis_dispatch_helper.h',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            helper_file_type  = 'layer_chassis_helper_header',
-            expandEnumerants = False)
+            helper_file_type  = 'layer_chassis_helper_header')
         ]
 
 
@@ -678,23 +424,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'layer_chassis_dispatch.cpp',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
-            emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            protectFeature    = False,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants = False)
+            emitExtensions    = emitExtensionsPat)
         ]
 
     # Options for layer chassis dispatch header file
@@ -704,23 +438,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'layer_chassis_dispatch.h',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
-            emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            protectFeature    = False,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants = False)
+            emitExtensions    = emitExtensionsPat)
         ]
 
     # Options for best practices code-generated source
@@ -730,23 +452,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'best_practices.cpp',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
-            emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            protectFeature    = False,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants  = False)
+            emitExtensions    = emitExtensionsPat)
         ]
 
     # Options for best_practices code-generated header
@@ -756,23 +466,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'best_practices.h',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
-            emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            protectFeature    = False,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants  = False)
+            emitExtensions    = emitExtensionsPat)
         ]
 
 # Create an API generator and corresponding generator options based on
@@ -784,22 +482,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'synchronization_validation_types.h',
             directory         = directory,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            protectFeature    = False,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants  = False,
             helper_file_type  = 'synchronization_helper_header',
             valid_usage_path  = args.scripts)
         ]
@@ -810,22 +497,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'synchronization_validation_types.cpp',
             directory         = directory,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            protectFeature    = False,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants  = False,
             helper_file_type  = 'synchronization_helper_source',
             valid_usage_path  = args.scripts)
         ]
@@ -837,23 +513,35 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'spirv_validation_helper.cpp',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = emitSpirvPat,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            protectFeature    = False,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants  = False)
+            emitFormats       = emitFormatsPat,
+            emitSpirv         = emitSpirvPat)
+        ]
+
+    # Options for spirv_grammar_helper code-generated source
+    # Only uses spirv grammar and not the vk.xml
+    genOpts['spirv_grammar_helper.cpp'] = [
+          SpirvGrammarHelperOutputGenerator,
+          SpirvGrammarHelperOutputGeneratorOptions(
+            conventions       = conventions,
+            filename          = 'spirv_grammar_helper.cpp',
+            directory         = directory,
+            grammar           = args.grammar)
+        ]
+
+    # Options for spirv_grammar_helper code-generated header
+    # Only uses spirv grammar and not the vk.xml
+    genOpts['spirv_grammar_helper.h'] = [
+          SpirvGrammarHelperOutputGenerator,
+          SpirvGrammarHelperOutputGeneratorOptions(
+            conventions       = conventions,
+            filename          = 'spirv_grammar_helper.h',
+            directory         = directory,
+            grammar           = args.grammar)
         ]
 
     # Options for command_validation code-generated header
@@ -863,23 +551,11 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'command_validation.cpp',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            protectFeature    = False,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants  = False,
             valid_usage_path  = args.scripts)
         ]
 
@@ -890,25 +566,34 @@ def makeGenOpts(args):
             conventions       = conventions,
             filename          = 'command_validation.h',
             directory         = directory,
-            genpath           = None,
-            apiname           = 'vulkan',
-            profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            emitSpirv         = None,
-            prefixText        = prefixStrings + vkPrefixStrings,
-            apicall           = 'VKAPI_ATTR ',
-            apientry          = 'VKAPI_CALL ',
-            apientryp         = 'VKAPI_PTR *',
-            alignFuncParam    = 48,
-            expandEnumerants = False,
             valid_usage_path  = args.scripts)
         ]
 
+    # Options for format_utils code-generated header
+    genOpts['vk_format_utils.cpp'] = [
+          FormatUtilsOutputGenerator,
+          FormatUtilsOutputGeneratorOptions(
+            conventions       = conventions,
+            filename          = 'vk_format_utils.cpp',
+            directory         = directory,
+            emitFormats       = emitFormatsPat)
+        ]
+
+    # generator for format_utils.h
+    genOpts['vk_format_utils.h'] = [
+          FormatUtilsOutputGenerator,
+          FormatUtilsOutputGeneratorOptions(
+            conventions       = conventions,
+            filename          = 'vk_format_utils.h',
+            directory         = directory,
+            emitFormats       = emitFormatsPat)
+        ]
+
 # Generate a target based on the options in the matching genOpts{} object.
 # This is encapsulated in a function so it can be profiled and/or timed.
 # The args parameter is an parsed argument object containing the following
@@ -937,6 +622,7 @@ def genTarget(args):
             write('* options.removeExtensions  =', options.removeExtensions, file=sys.stderr)
             write('* options.emitExtensions    =', options.emitExtensions, file=sys.stderr)
             write('* options.emitSpirv         =', options.emitSpirv, file=sys.stderr)
+            write('* options.emitFormats       =', options.emitFormats, file=sys.stderr)
 
         gen = createGenerator(errFile=errWarn,
                               warnFile=errWarn,
@@ -968,12 +654,18 @@ if __name__ == '__main__':
     parser.add_argument('-emitExtensions', action='append',
                         default=[],
                         help='Specify an extension or extensions to emit in targets')
+    parser.add_argument('-warnExtensions', action='append',
+                        default=[],
+                        help='Specify an extension with partial support. Warning will be log if it is enabled')
     parser.add_argument('-feature', action='append',
                         default=[],
                         help='Specify a core API feature name or names to add to targets')
     parser.add_argument('-emitSpirv', action='append',
                         default=[],
                         help='Specify spirv extensions or capabilities to emit in targets')
+    parser.add_argument('-emitFormats', action='append',
+                        default=[],
+                        help='Specify spirv extensions or capabilities to emit in targets')
     parser.add_argument('-debug', action='store_true',
                         help='Enable debugging')
     parser.add_argument('-dump', action='store_true',
@@ -991,6 +683,9 @@ if __name__ == '__main__':
     parser.add_argument('-registry', action='store',
                         default='vk.xml',
                         help='Use specified registry file instead of vk.xml')
+    parser.add_argument('-grammar', action='store',
+                        default='spirv.core.grammar.json',
+                        help='Use specified grammar file instead of spirv.core.grammar.json')
     parser.add_argument('-time', action='store_true',
                         help='Enable timing')
     parser.add_argument('-validate', action='store_true',
@@ -1037,7 +732,9 @@ if __name__ == '__main__':
     from lvt_file_generator import LvtFileOutputGenerator, LvtFileOutputGeneratorOptions
     from best_practices_generator import BestPracticesOutputGenerator, BestPracticesOutputGeneratorOptions
     from spirv_validation_generator import SpirvValidationHelperOutputGenerator, SpirvValidationHelperOutputGeneratorOptions
+    from spirv_grammar_generator import SpirvGrammarHelperOutputGenerator, SpirvGrammarHelperOutputGeneratorOptions
     from command_validation_generator import CommandValidationOutputGenerator, CommandValidationOutputGeneratorOptions
+    from format_utils_generator import FormatUtilsOutputGenerator, FormatUtilsOutputGeneratorOptions
 
     # Temporary workaround for vkconventions python2 compatibility
     import abc; abc.ABC = abc.ABCMeta('ABC', (object,), {})
