@@ -147,6 +147,7 @@ class ObjectTrackerOutputGenerator(OutputGenerator):
             'vkCreateInstance',
             'vkCreateDevice',
             'vkEnumeratePhysicalDevices',
+            #'vkEnumeratePhysicalDeviceGroups',            
             'vkGetPhysicalDeviceQueueFamilyProperties',
             'vkGetPhysicalDeviceQueueFamilyProperties2',
             'vkGetPhysicalDeviceQueueFamilyProperties2KHR',
@@ -180,6 +181,8 @@ class ObjectTrackerOutputGenerator(OutputGenerator):
             'vkCmdBuildAccelerationStructuresKHR',
             'vkCmdBuildAccelerationStructuresIndirectKHR',
             'vkBuildAccelerationStructuresKHR',
+            'vkCreateRayTracingPipelinesKHR',
+            'vkExportMetalObjectsEXT'
             ]
         # These VUIDS are not implicit, but are best handled in this layer. Codegen for vkDestroy calls will generate a key
         # which is translated here into a good VU.  Saves ~40 checks.
@@ -225,6 +228,10 @@ class ObjectTrackerOutputGenerator(OutputGenerator):
             "VkComputePipelineCreateInfo-basePipelineHandle": "\"VUID-VkComputePipelineCreateInfo-flags-00697\"",
             "VkRayTracingPipelineCreateInfoNV-basePipelineHandle": "\"VUID-VkRayTracingPipelineCreateInfoNV-flags-03421\"",
 			"VkRayTracingPipelineCreateInfoKHR-basePipelineHandle": "\"VUID-VkRayTracingPipelineCreateInfoKHR-flags-03421\"",
+            "VkAccelerationStructureKHR-accelerationStructure-compatalloc": "\"VUID-vkDestroyAccelerationStructureKHR-accelerationStructure-02443\"",
+            "VkAccelerationStructureKHR-accelerationStructure-nullalloc": "\"VUID-vkDestroyAccelerationStructureKHR-accelerationStructure-02444\"",
+            "VkAccelerationStructureNV-accelerationStructure-compatalloc": "\"VUID-vkDestroyAccelerationStructureNV-accelerationStructure-03753\"",
+            "VkAccelerationStructureNV-accelerationStructure-nullalloc": "\"VUID-vkDestroyAccelerationStructureNV-accelerationStructure-03754\"",
            }
 
         # Commands shadowed by interface functions and are not implemented
@@ -738,8 +745,27 @@ class ObjectTrackerOutputGenerator(OutputGenerator):
                 indent = self.decIndent(indent)
                 create_obj_code += '%s}\n' % indent
             indent = self.decIndent(indent)
+        # Physical device groups are not handles, but a set of handles, they need to be tracked as well
+        elif handle_type.text == 'VkPhysicalDeviceGroupProperties':
+            create_obj_code += f"""{indent}if ({cmd_info[-1].name}) {{
+{indent}{indent}for (uint32_t device_group_index = 0; device_group_index < *{cmd_info[-2].name}; device_group_index++) {{
+{indent}{indent}{indent}PostCallRecordEnumeratePhysicalDevices({cmd_info[0].name}, &{cmd_info[-1].name}[device_group_index].physicalDeviceCount, {cmd_info[-1].name}[device_group_index].physicalDevices, VK_SUCCESS);
+{indent}{indent}}}
+{indent}}}\n"""
 
         return create_obj_code
+
+    def get_alloc_vuid(self, param_name, param_type, alloc_type):
+        lookup_string = '%s-%s' %(param_name, alloc_type)
+        vuid = self.manual_vuids.get(lookup_string, None)
+        if vuid is not None:
+            return vuid
+        lookup_string = '%s-%s-%s' %(param_type, param_name, alloc_type)
+        vuid = self.manual_vuids.get(lookup_string, None)
+        if vuid is not None:
+            return vuid
+        return "kVUIDUndefined"
+
     #
     # Generate source for destroying a non-dispatchable object
     def generate_destroy_object_code(self, indent, proto, cmd_info):
@@ -757,10 +783,8 @@ class ObjectTrackerOutputGenerator(OutputGenerator):
                 allocator = 'nullptr'
             else:
                 param = -2
-            compatalloc_vuid_string = '%s-compatalloc' % cmd_info[param].name
-            nullalloc_vuid_string = '%s-nullalloc' % cmd_info[param].name
-            compatalloc_vuid = self.manual_vuids.get(compatalloc_vuid_string, "kVUIDUndefined")
-            nullalloc_vuid = self.manual_vuids.get(nullalloc_vuid_string, "kVUIDUndefined")
+            compatalloc_vuid = self.get_alloc_vuid(cmd_info[param].name, cmd_info[param].type, "compatalloc")
+            nullalloc_vuid = self.get_alloc_vuid(cmd_info[param].name, cmd_info[param].type, "nullalloc")
             if cmd_info[param].type in self.handle_types:
                 if object_array == True:
                     # This API is freeing an array of handles -- add loop control
@@ -799,7 +823,7 @@ class ObjectTrackerOutputGenerator(OutputGenerator):
                 parent_vuid = self.GetVuid(parent_name, 'commonparent')
 
         if obj_count is not None:
-            pre_call_code += '%sif (%s%s) {\n' % (indent, prefix, obj_name)
+            pre_call_code += '%sif ((%s > 0) && (%s%s)) {\n' % (indent, obj_count, prefix, obj_name)
             indent = self.incIndent(indent)
             pre_call_code += '%sfor (uint32_t %s = 0; %s < %s; ++%s) {\n' % (indent, index, index, obj_count, index)
             indent = self.incIndent(indent)
@@ -993,7 +1017,7 @@ class ObjectTrackerOutputGenerator(OutputGenerator):
             (pre_call_validate, pre_call_record, post_call_record) = self.generate_wrapping_code(cmdinfo.elem)
 
             feature_extra_protect = cmddata.extra_protect
-            if (feature_extra_protect is not None):
+            if (feature_extra_protect is not None and not (cmdname in self.no_autogen_list and 'object_tracker.cpp' in self.genOpts.filename)):
                 self.appendSection('command', '')
                 self.appendSection('command', '#ifdef '+ feature_extra_protect)
                 self.prototypes += [ '#ifdef %s' % feature_extra_protect ]
@@ -1064,9 +1088,13 @@ class ObjectTrackerOutputGenerator(OutputGenerator):
 
                     if result_type.text == 'VkResult':
                         post_cr_func_decl = post_cr_func_decl.replace(')', ',\n    VkResult                                    result)')
+                        failure_condition = 'result != VK_SUCCESS'
+                        # VK_INCOMPLETE is considered a success
+                        if 'EnumeratePhysicalDeviceGroups' in cmdname:
+                            failure_condition += ' && result != VK_INCOMPLETE'
                         # The two createpipelines APIs may create on failure -- skip the success result check
                         if 'CreateGraphicsPipelines' not in cmdname and 'CreateComputePipelines' not in cmdname and 'CreateRayTracingPipelines' not in cmdname:
-                            post_cr_func_decl = post_cr_func_decl.replace('{', '{\n    if (result != VK_SUCCESS) return;')
+                            post_cr_func_decl = post_cr_func_decl.replace('{', '{\n    if (%s) return;' % failure_condition)
                     elif result_type.text == 'VkDeviceAddress':
                         post_cr_func_decl = post_cr_func_decl.replace(')', ',\n    VkDeviceAddress                             result)')
                     self.appendSection('command', post_cr_func_decl)
@@ -1074,6 +1102,6 @@ class ObjectTrackerOutputGenerator(OutputGenerator):
                     self.appendSection('command', post_call_record)
                     self.appendSection('command', '}')
 
-            if (feature_extra_protect is not None):
+            if (feature_extra_protect is not None and not (cmdname in self.no_autogen_list and 'object_tracker.cpp' in self.genOpts.filename)):
                 self.appendSection('command', '#endif // '+ feature_extra_protect)
                 self.prototypes += [ '#endif' ]
