@@ -33,7 +33,7 @@
 #include <vulkan/vulkan.h>
 #endif
 
-#include "layers/vk_device_profile_api_layer.h"
+#include "layers/vk_lunarg_device_profile_api_layer.h"
 #include "vk_layer_settings_ext.h"
 
 #if defined(ANDROID)
@@ -176,6 +176,14 @@ static const char bindStateFragUniformShaderText[] = R"glsl(
     }
 )glsl";
 
+static char const bindStateFragSubpassLoadInputText[] = R"glsl(
+        #version 450
+        layout(input_attachment_index=0, set=0, binding=0) uniform subpassInput x;
+        void main() {
+           vec4 color = subpassLoad(x);
+        }
+    )glsl";
+
 // Static arrays helper
 template <class ElementT, size_t array_size>
 size_t size(ElementT (&)[array_size]) {
@@ -213,12 +221,6 @@ VkImageViewCreateInfo SafeSaneImageViewCreateInfo(const VkImageObj &image, VkFor
 // Helper for checking createRenderPass2 support and adding related extensions.
 bool CheckCreateRenderPass2Support(VkRenderFramework *renderFramework, std::vector<const char *> &device_extension_names);
 
-// Helper for checking descriptor_indexing support and adding related extensions.
-bool CheckDescriptorIndexingSupportAndInitFramework(VkRenderFramework *renderFramework,
-                                                    std::vector<const char *> &instance_extension_names,
-                                                    std::vector<const char *> &device_extension_names,
-                                                    VkValidationFeaturesEXT *features, void *userData);
-
 // Helper for checking timeline semaphore support and initializing
 bool CheckTimelineSemaphoreSupportAndInitState(VkRenderFramework *renderFramework);
 
@@ -258,17 +260,30 @@ class VkLayerTest : public VkRenderFramework {
 
     void Init(VkPhysicalDeviceFeatures *features = nullptr, VkPhysicalDeviceFeatures2 *features2 = nullptr,
               const VkCommandPoolCreateFlags flags = 0, void *instance_pnext = nullptr);
-    bool AddSurfaceInstanceExtension();
-    bool AddSwapchainDeviceExtension();
+    void AddSurfaceExtension();
     VkCommandBufferObj *CommandBuffer();
     void OOBRayTracingShadersTestBody(bool gpu_assisted);
 
+    template <typename Features>
+    VkPhysicalDeviceFeatures2 GetPhysicalDeviceFeatures2(Features &feature_query) {
+        auto features2 = LvlInitStruct<VkPhysicalDeviceFeatures2>(&feature_query);
+        return GetPhysicalDeviceFeatures2(features2);
+    }
+
+    template <typename Properties>
+    VkPhysicalDeviceProperties2 GetPhysicalDeviceProperties2(Properties &props_query) {
+        auto props2 = LvlInitStruct<VkPhysicalDeviceProperties2>(&props_query);
+        return GetPhysicalDeviceProperties2(props2);
+    }
+
+    bool IsDriver(VkDriverId driver_id);
+
   protected:
     uint32_t m_instance_api_version = 0;
     uint32_t m_target_api_version = 0;
     bool m_enableWSI;
 
-    uint32_t SetTargetApiVersion(uint32_t target_api_version);
+    void SetTargetApiVersion(uint32_t target_api_version);
     uint32_t DeviceValidationVersion();
     bool LoadDeviceProfileLayer(
         PFN_vkSetPhysicalDeviceFormatPropertiesEXT &fpvkSetPhysicalDeviceFormatPropertiesEXT,
@@ -278,10 +293,19 @@ class VkLayerTest : public VkRenderFramework {
         PFN_vkGetOriginalPhysicalDeviceFormatProperties2EXT &fpvkGetOriginalPhysicalDeviceFormatProperties2EXT);
     bool LoadDeviceProfileLayer(PFN_vkSetPhysicalDeviceLimitsEXT &fpvkSetPhysicalDeviceLimitsEXT,
                                 PFN_vkGetOriginalPhysicalDeviceLimitsEXT &fpvkGetOriginalPhysicalDeviceLimitsEXT);
+    bool LoadDeviceProfileLayer(PFN_vkSetPhysicalDeviceFeaturesEXT &fpvkSetPhysicalDeviceFeaturesEXT,
+                                PFN_vkGetOriginalPhysicalDeviceFeaturesEXT &fpvkGetOriginalPhysicalDeviceFeaturesEXT);
+    bool LoadDeviceProfileLayer(PFN_VkSetPhysicalDeviceProperties2EXT &fpvkSetPhysicalDeviceProperties2EXT);
 
     VkLayerTest();
 };
 
+template <>
+VkPhysicalDeviceFeatures2 VkLayerTest::GetPhysicalDeviceFeatures2(VkPhysicalDeviceFeatures2 &feature_query);
+
+template <>
+VkPhysicalDeviceProperties2 VkLayerTest::GetPhysicalDeviceProperties2(VkPhysicalDeviceProperties2 &props2);
+
 class VkPositiveLayerTest : public VkLayerTest {
   public:
   protected:
@@ -310,6 +334,7 @@ class VkArmBestPracticesLayerTest : public VkBestPracticesLayerTest {
     VkFramebuffer CreateFramebuffer(const uint32_t width, const uint32_t height, VkImageView image_view, VkRenderPass renderpass);
     VkSampler CreateDefaultSampler();
 };
+class VkNvidiaBestPracticesLayerTest : public VkBestPracticesLayerTest {};
 
 class VkWsiEnabledLayerTest : public VkLayerTest {
   public:
@@ -319,11 +344,12 @@ class VkWsiEnabledLayerTest : public VkLayerTest {
 
 class VkGpuAssistedLayerTest : public VkLayerTest {
   public:
-    bool InitGpuAssistedFramework(bool request_descriptor_indexing);
+    VkValidationFeaturesEXT GetValidationFeatures();
     void ShaderBufferSizeTest(VkDeviceSize buffer_size, VkDeviceSize binding_offset, VkDeviceSize binding_range,
                               VkDescriptorType descriptor_type, const char *fragment_shader, const char *expected_error);
 
   protected:
+    bool CanEnableGpuAV();
 };
 
 class VkDebugPrintfTest : public VkLayerTest {
@@ -335,7 +361,7 @@ class VkDebugPrintfTest : public VkLayerTest {
 
 class VkSyncValTest : public VkLayerTest {
   public:
-    void InitSyncValFramework();
+    void InitSyncValFramework(bool enable_queue_submit_validation = false);
 
   protected:
     VkValidationFeatureEnableEXT enables_[1] = {VK_VALIDATION_FEATURE_ENABLE_SYNCHRONIZATION_VALIDATION_EXT};
@@ -458,7 +484,7 @@ struct CreatePipelineHelper {
     VkPipelineDynamicStateCreateInfo dyn_state_ci_ = {};
     VkPipelineRasterizationStateCreateInfo rs_state_ci_ = {};
     VkPipelineRasterizationLineStateCreateInfoEXT line_state_ci_ = {};
-    VkPipelineColorBlendAttachmentState cb_attachments_ = {};
+    std::vector<VkPipelineColorBlendAttachmentState> cb_attachments_ = {};
     VkPipelineColorBlendStateCreateInfo cb_ci_ = {};
     VkPipelineDepthStencilStateCreateInfo ds_ci_ = {};
     VkGraphicsPipelineCreateInfo gp_ci_ = {};
@@ -468,7 +494,8 @@ struct CreatePipelineHelper {
     std::unique_ptr<VkShaderObj> vs_;
     std::unique_ptr<VkShaderObj> fs_;
     VkLayerTest &layer_test_;
-    CreatePipelineHelper(VkLayerTest &test);
+    layer_data::optional<VkGraphicsPipelineLibraryCreateInfoEXT> gpl_info;
+    CreatePipelineHelper(VkLayerTest &test, uint32_t color_attachments_count = 1u);
     ~CreatePipelineHelper();
 
     void InitDescriptorSetInfo();
@@ -478,6 +505,7 @@ struct CreatePipelineHelper {
     void InitViewportInfo();
     void InitDynamicStateInfo();
     void InitShaderInfo();
+    void ResetShaderInfo(const char *vertex_shader_text, const char *fragment_shader_text);
     void InitRasterizationInfo();
     void InitLineRasterizationInfo();
     void InitBlendStateInfo();
@@ -490,16 +518,32 @@ struct CreatePipelineHelper {
     // TDB -- add control for optional and/or additional initialization
     void InitInfo();
     void InitState();
+    void InitPipelineCache();
     void LateBindPipelineInfo();
     VkResult CreateGraphicsPipeline(bool implicit_destroy = true, bool do_late_bind = true);
 
+    void InitVertexInputLibInfo(void *p_next = nullptr);
+
+    template <typename StageContainer>
+    void InitPreRasterLibInfoFromContainer(const StageContainer &stages, void *p_next = nullptr) {
+        InitPreRasterLibInfo(static_cast<uint32_t>(stages.size()), stages.data(), p_next);
+    }
+    void InitPreRasterLibInfo(uint32_t count, const VkPipelineShaderStageCreateInfo *info, void *p_next = nullptr);
+
+    template <typename StageContainer>
+    void InitFragmentLibInfoFromContainer(const StageContainer &stages, void *p_next = nullptr) {
+        InitFragmentLibInfo(static_cast<uint32_t>(stages.size()), stages.data(), p_next);
+    }
+    void InitFragmentLibInfo(uint32_t count, const VkPipelineShaderStageCreateInfo *info, void *p_next = nullptr);
+
+    void InitFragmentOutputLibInfo(void *p_next = nullptr);
+
     // Helper function to create a simple test case (positive or negative)
     //
     // info_override can be any callable that takes a CreatePipelineHeper &
     // flags, error can be any args accepted by "SetDesiredFailure".
     template <typename Test, typename OverrideFunc, typename Error>
-    static void OneshotTest(Test &test, const OverrideFunc &info_override, const VkFlags flags, const std::vector<Error> &errors,
-                            bool positive_test = false) {
+    static void OneshotTest(Test &test, const OverrideFunc &info_override, const VkFlags flags, const std::vector<Error> &errors) {
         CreatePipelineHelper helper(test);
         helper.InitInfo();
         info_override(helper);
@@ -508,17 +552,19 @@ struct CreatePipelineHelper {
         for (const auto &error : errors) test.Monitor().SetDesiredFailureMsg(flags, error);
         helper.CreateGraphicsPipeline();
 
-        if (positive_test || (errors.size() == 0)) {
-            test.Monitor().VerifyNotFound();
-        } else {
+        if (!errors.empty()) {
             test.Monitor().VerifyFound();
         }
     }
 
     template <typename Test, typename OverrideFunc, typename Error>
-    static void OneshotTest(Test &test, const OverrideFunc &info_override, const VkFlags flags, Error error,
-                            bool positive_test = false) {
-        OneshotTest(test, info_override, flags, std::vector<Error>(1, error), positive_test);
+    static void OneshotTest(Test &test, const OverrideFunc &info_override, const VkFlags flags, Error error) {
+        OneshotTest(test, info_override, flags, std::vector<Error>(1, error));
+    }
+
+    template <typename Test, typename OverrideFunc>
+    static void OneshotTest(Test &test, const OverrideFunc &info_override, const VkFlags flags) {
+        OneshotTest(test, info_override, flags, std::vector<std::string>{});
     }
 };
 
@@ -547,6 +593,7 @@ struct CreateComputePipelineHelper {
     // TDB -- add control for optional and/or additional initialization
     void InitInfo();
     void InitState();
+    void InitPipelineCache();
     void LateBindPipelineInfo();
     VkResult CreateComputePipeline(bool implicit_destroy = true, bool do_late_bind = true);
 
@@ -570,17 +617,19 @@ struct CreateComputePipelineHelper {
         for (const auto &error : errors) test.Monitor().SetDesiredFailureMsg(flags, error);
         helper.CreateComputePipeline();
 
-        if (positive_test) {
-            test.Monitor().VerifyNotFound();
-        } else {
+        if (!errors.empty()) {
             test.Monitor().VerifyFound();
         }
     }
 
     template <typename Test, typename OverrideFunc, typename Error>
-    static void OneshotTest(Test &test, const OverrideFunc &info_override, const VkFlags flags, Error error,
-                            bool positive_test = false) {
-        OneshotTest(test, info_override, flags, std::vector<Error>(1, error), positive_test);
+    static void OneshotTest(Test &test, const OverrideFunc &info_override, const VkFlags flags, Error error) {
+        OneshotTest(test, info_override, flags, std::vector<Error>(1, error));
+    }
+
+    template <typename Test, typename OverrideFunc>
+    static void OneshotTest(Test &test, const OverrideFunc &info_override, const VkFlags flags) {
+        OneshotTest(test, info_override, flags, std::vector<std::string>{});
     }
 };
 
@@ -623,6 +672,7 @@ struct CreateNVRayTracingPipelineHelper {
     void InitPipelineCacheInfo();
     void InitInfo(bool isKHR = false);
     void InitState();
+    void InitPipelineCache();
     void LateBindPipelineInfo(bool isKHR = false);
     VkResult CreateNVRayTracingPipeline(bool implicit_destroy = true, bool do_late_bind = true);
     VkResult CreateKHRRayTracingPipeline(bool implicit_destroy = true, bool do_late_bind = true);
@@ -655,9 +705,7 @@ struct CreateNVRayTracingPipelineHelper {
         info_override(helper);
         helper.InitState();
 
-        test.Monitor().ExpectSuccess(message_flag_mask);
         ASSERT_VK_SUCCESS(helper.CreateNVRayTracingPipeline());
-        test.Monitor().VerifyNotFound();
     }
 };
 
@@ -696,15 +744,6 @@ class ExtensionChain {
 };
 }  // namespace chain_util
 
-// PushDescriptorProperties helper
-VkPhysicalDevicePushDescriptorPropertiesKHR GetPushDescriptorProperties(VkInstance instance, VkPhysicalDevice gpu);
-
-// Subgroup properties helper
-VkPhysicalDeviceSubgroupProperties GetSubgroupProperties(VkInstance instance, VkPhysicalDevice gpu);
-
-// Descriptor Indexing properties helper
-VkPhysicalDeviceDescriptorIndexingProperties GetDescriptorIndexingProperties(VkInstance instance, VkPhysicalDevice gpu);
-
 class BarrierQueueFamilyBase {
   public:
     struct QueueFamilyObjs {
@@ -749,8 +788,13 @@ class BarrierQueueFamilyTestHelper : public BarrierQueueFamilyBase {
     void Init(std::vector<uint32_t> *families, bool image_memory = true, bool buffer_memory = true);
 
     void operator()(std::string img_err, std::string buf_err = "", uint32_t src = VK_QUEUE_FAMILY_IGNORED,
-                    uint32_t dst = VK_QUEUE_FAMILY_IGNORED, bool positive = false,
-                    uint32_t queue_family_index = kInvalidQueueFamily, Modifier mod = Modifier::NONE);
+                    uint32_t dst = VK_QUEUE_FAMILY_IGNORED, uint32_t queue_family_index = kInvalidQueueFamily,
+                    Modifier mod = Modifier::NONE);
+
+    void operator()(uint32_t src = VK_QUEUE_FAMILY_IGNORED, uint32_t dst = VK_QUEUE_FAMILY_IGNORED,
+                    uint32_t queue_family_index = kInvalidQueueFamily, Modifier mod = Modifier::NONE) {
+        (*this)("", "", src, dst, queue_family_index, mod);
+    }
 
     VkImageMemoryBarrier image_barrier_;
     VkBufferMemoryBarrier buffer_barrier_;
@@ -763,8 +807,13 @@ class Barrier2QueueFamilyTestHelper : public BarrierQueueFamilyBase {
     void Init(std::vector<uint32_t> *families, bool image_memory = true, bool buffer_memory = true);
 
     void operator()(std::string img_err, std::string buf_err = "", uint32_t src = VK_QUEUE_FAMILY_IGNORED,
-                    uint32_t dst = VK_QUEUE_FAMILY_IGNORED, bool positive = false,
-                    uint32_t queue_family_index = kInvalidQueueFamily, Modifier mod = Modifier::NONE);
+                    uint32_t dst = VK_QUEUE_FAMILY_IGNORED, uint32_t queue_family_index = kInvalidQueueFamily,
+                    Modifier mod = Modifier::NONE);
+
+    void operator()(uint32_t src = VK_QUEUE_FAMILY_IGNORED, uint32_t dst = VK_QUEUE_FAMILY_IGNORED,
+                    uint32_t queue_family_index = kInvalidQueueFamily, Modifier mod = Modifier::NONE) {
+        (*this)("", "", src, dst, queue_family_index, mod);
+    }
 
     VkImageMemoryBarrier2KHR image_barrier_;
     VkBufferMemoryBarrier2KHR buffer_barrier_;
@@ -782,21 +831,21 @@ VKAPI_ATTR VkBool32 VKAPI_CALL DebugUtilsCallback(VkDebugUtilsMessageSeverityFla
                                                   const VkDebugUtilsMessengerCallbackDataEXT *pCallbackData, void *pUserData);
 
 #if GTEST_IS_THREADSAFE
-struct thread_data_struct {
+struct ThreadTestData {
     VkCommandBuffer commandBuffer;
     VkDevice device;
     VkEvent event;
     VkDescriptorSet descriptorSet;
     VkBuffer buffer;
     uint32_t binding;
-    bool *bailout;
+    std::atomic<bool> *bailout;
 };
 
-extern "C" void *AddToCommandBuffer(void *arg);
-extern "C" void *UpdateDescriptor(void *arg);
+void AddToCommandBuffer(ThreadTestData *);
+void UpdateDescriptor(ThreadTestData *);
 #endif  // GTEST_IS_THREADSAFE
 
-extern "C" void *ReleaseNullFence(void *arg);
+void ReleaseNullFence(ThreadTestData *);
 
 void TestRenderPassCreate(ErrorMonitor *error_monitor, const VkDevice device, const VkRenderPassCreateInfo *create_info,
                           bool rp2_supported, const char *rp1_vuid, const char *rp2_vuid);
@@ -852,14 +901,11 @@ void CreateBufferViewTest(VkLayerTest &test, const VkBufferViewCreateInfo *pCrea
 
 void CreateImageViewTest(VkLayerTest &test, const VkImageViewCreateInfo *pCreateInfo, std::string code = "");
 
-bool InitFrameworkForRayTracingTest(VkRenderFramework *renderFramework, bool isKHR,
-                                    std::vector<const char *> &instance_extension_names,
-                                    std::vector<const char *> &device_extension_names, void *user_data,
-                                    bool need_gpu_validation = false, bool need_push_descriptors = false,
-                                    bool deferred_state_init = false, VkPhysicalDeviceFeatures2KHR *features2 = nullptr);
+bool InitFrameworkForRayTracingTest(VkRenderFramework *framework, bool is_khr, VkPhysicalDeviceFeatures2KHR *features2 = nullptr,
+                                    VkValidationFeaturesEXT *enabled_features = nullptr, bool mockicd_valid = false);
 
 void GetSimpleGeometryForAccelerationStructureTests(const VkDeviceObj &device, VkBufferObj *vbo, VkBufferObj *ibo,
-                                                    VkGeometryNV *geometry);
+                                                    VkGeometryNV *geometry, VkDeviceSize offset = 0, bool buffer_device_address = false);
 
 void print_android(const char *c);
 #endif  // VKLAYERTEST_H
