@@ -1,7 +1,7 @@
 /*
- * Copyright (c) 2015-2020 The Khronos Group Inc.
- * Copyright (c) 2015-2020 Valve Corporation
- * Copyright (c) 2015-2020 LunarG, Inc.
+ * Copyright (c) 2015-2022 The Khronos Group Inc.
+ * Copyright (c) 2015-2022 Valve Corporation
+ * Copyright (c) 2015-2022 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -29,19 +29,28 @@
 
 #include "test_common.h"
 #include "vk_typemap_helper.h"
+#include "vk_format_utils.h"
 
 namespace {
 
-#define NON_DISPATCHABLE_HANDLE_INIT(create_func, dev, ...)                              \
-    do {                                                                                 \
-        handle_type handle;                                                              \
-        if (EXPECT(create_func(dev.handle(), __VA_ARGS__, NULL, &handle) == VK_SUCCESS)) \
-            NonDispHandle::init(dev.handle(), handle);                                   \
+#define NON_DISPATCHABLE_HANDLE_INIT(create_func, dev, ...)                                                 \
+    do {                                                                                                    \
+        handle_type handle;                                                                                 \
+        auto result = create_func(dev.handle(), __VA_ARGS__, NULL, &handle);                                \
+        if (EXPECT((result == VK_SUCCESS) || (result == VK_ERROR_VALIDATION_FAILED_EXT) ||                  \
+                   (result == VK_ERROR_OUT_OF_DEVICE_MEMORY) || (result == VK_ERROR_OUT_OF_HOST_MEMORY))) { \
+            if (result == VK_SUCCESS) {                                                                     \
+                NonDispHandle::init(dev.handle(), handle);                                                  \
+            }                                                                                               \
+        }                                                                                                   \
     } while (0)
 
-#define NON_DISPATCHABLE_HANDLE_DTOR(cls, destroy_func)            \
-    cls::~cls() NOEXCEPT {                                         \
-        if (initialized()) destroy_func(device(), handle(), NULL); \
+#define NON_DISPATCHABLE_HANDLE_DTOR(cls, destroy_func) \
+    cls::~cls() noexcept {                              \
+        if (initialized()) {                            \
+            destroy_func(device(), handle(), NULL);     \
+            handle_ = VK_NULL_HANDLE;                   \
+        }                                               \
     }
 
 #define STRINGIFY(x) #x
@@ -104,23 +113,20 @@ VkPhysicalDeviceFeatures PhysicalDevice::features() const {
  */
 std::vector<VkLayerProperties> GetGlobalLayers() {
     VkResult err;
-    uint32_t layer_count;
-    std::vector<VkLayerProperties> layers;
-
-    do {
-        err = vk::EnumerateInstanceLayerProperties(&layer_count, nullptr);
-        assert(!err);
-        if (err || 0 == layer_count) return {};
-
-        layers.resize(layer_count);
+    uint32_t layer_count = 32;
+    std::vector<VkLayerProperties> layers(layer_count);
+    for (;;) {
         err = vk::EnumerateInstanceLayerProperties(&layer_count, layers.data());
-    } while (VK_INCOMPLETE == err);
-
-    assert(!err);
-    if (err) return {};
-    layers.resize(layer_count);
-
-    return layers;
+        if (err == VK_SUCCESS) {
+            layers.resize(layer_count);
+            return layers;
+        } else if (err == VK_INCOMPLETE) {
+            layer_count *= 2;  // wasn't enough space, increase it
+            layers.resize(layer_count);
+        } else {
+            return {};
+        }
+    }
 }
 
 /*
@@ -135,23 +141,20 @@ std::vector<VkExtensionProperties> GetGlobalExtensions() { return GetGlobalExten
  */
 std::vector<VkExtensionProperties> GetGlobalExtensions(const char *pLayerName) {
     VkResult err;
-    uint32_t extension_count;
-    std::vector<VkExtensionProperties> extensions;
-
-    do {
-        err = vk::EnumerateInstanceExtensionProperties(nullptr, &extension_count, nullptr);
-        assert(!err);
-        if (err || 0 == extension_count) return {};
-
-        extensions.resize(extension_count);
-        err = vk::EnumerateInstanceExtensionProperties(nullptr, &extension_count, extensions.data());
-    } while (VK_INCOMPLETE == err);
-
-    assert(!err);
-    if (err) return {};
-    extensions.resize(extension_count);
-
-    return extensions;
+    uint32_t extension_count = 32;
+    std::vector<VkExtensionProperties> extensions(extension_count);
+    for (;;) {
+        err = vk::EnumerateInstanceExtensionProperties(pLayerName, &extension_count, extensions.data());
+        if (err == VK_SUCCESS) {
+            extensions.resize(extension_count);
+            return extensions;
+        } else if (err == VK_INCOMPLETE) {
+            extension_count *= 2;  // wasn't enough space, increase it
+            extensions.resize(extension_count);
+        } else {
+            return {};
+        }
+    }
 }
 
 /*
@@ -160,20 +163,20 @@ std::vector<VkExtensionProperties> GetGlobalExtensions(const char *pLayerName) {
  */
 std::vector<VkExtensionProperties> PhysicalDevice::extensions(const char *pLayerName) const {
     VkResult err;
-    uint32_t extension_count;
-    std::vector<VkExtensionProperties> extensions;
-    do {
-        err = vk::EnumerateDeviceExtensionProperties(handle(), pLayerName, &extension_count, nullptr);
-        if (err || 0 == extension_count) return {};
-
-        extensions.resize(extension_count);
+    uint32_t extension_count = 512;
+    std::vector<VkExtensionProperties> extensions(extension_count);
+    for (;;) {
         err = vk::EnumerateDeviceExtensionProperties(handle(), pLayerName, &extension_count, extensions.data());
-    } while (VK_INCOMPLETE == err);
-
-    if (err) return {};
-    extensions.resize(extension_count);
-
-    return extensions;
+        if (err == VK_SUCCESS) {
+            extensions.resize(extension_count);
+            return extensions;
+        } else if (err == VK_INCOMPLETE) {
+            extension_count *= 2;  // wasn't enough space, increase it
+            extensions.resize(extension_count);
+        } else {
+            return {};
+        }
+    }
 }
 
 bool PhysicalDevice::set_memory_type(const uint32_t type_bits, VkMemoryAllocateInfo *info, const VkFlags properties,
@@ -184,7 +187,8 @@ bool PhysicalDevice::set_memory_type(const uint32_t type_bits, VkMemoryAllocateI
         if ((type_mask & 1) == 1) {
             // Type is available, does it match user properties?
             if ((memory_properties_.memoryTypes[i].propertyFlags & properties) == properties &&
-                (memory_properties_.memoryTypes[i].propertyFlags & forbid) == 0) {
+                (memory_properties_.memoryTypes[i].propertyFlags & forbid) == 0 &&
+                (memory_properties_.memoryHeaps[memory_properties_.memoryTypes[i].heapIndex].size >= info->allocationSize)) {
                 info->memoryTypeIndex = i;
                 return true;
             }
@@ -200,20 +204,20 @@ bool PhysicalDevice::set_memory_type(const uint32_t type_bits, VkMemoryAllocateI
  */
 std::vector<VkLayerProperties> PhysicalDevice::layers() const {
     VkResult err;
-    uint32_t layer_count;
-    std::vector<VkLayerProperties> layers;
-    do {
-        err = vk::EnumerateDeviceLayerProperties(handle(), &layer_count, nullptr);
-        if (err || 0 == layer_count) return {};
-
-        layers.resize(layer_count);
+    uint32_t layer_count = 32;
+    std::vector<VkLayerProperties> layers(layer_count);
+    for (;;) {
         err = vk::EnumerateDeviceLayerProperties(handle(), &layer_count, layers.data());
-    } while (VK_INCOMPLETE == err);
-
-    if (err) return {};
-    layers.resize(layer_count);
-
-    return layers;
+        if (err == VK_SUCCESS) {
+            layers.resize(layer_count);
+            return layers;
+        } else if (err == VK_INCOMPLETE) {
+            layer_count *= 2;  // wasn't enough space, increase it
+            layers.resize(layer_count);
+        } else {
+            return {};
+        }
+    }
 }
 
 QueueCreateInfoArray::QueueCreateInfoArray(const std::vector<VkQueueFamilyProperties> &queue_props)
@@ -222,9 +226,7 @@ QueueCreateInfoArray::QueueCreateInfoArray(const std::vector<VkQueueFamilyProper
 
     for (uint32_t i = 0; i < (uint32_t)queue_props.size(); ++i) {
         if (queue_props[i].queueCount > 0) {
-            VkDeviceQueueCreateInfo qi = {};
-            qi.sType = VK_STRUCTURE_TYPE_DEVICE_QUEUE_CREATE_INFO;
-            qi.pNext = NULL;
+            VkDeviceQueueCreateInfo qi = LvlInitStruct<VkDeviceQueueCreateInfo>();
             qi.queueFamilyIndex = i;
             qi.queueCount = queue_props[i].queueCount;
             queue_priorities_.emplace_back(qi.queueCount, 0.0f);
@@ -234,7 +236,7 @@ QueueCreateInfoArray::QueueCreateInfoArray(const std::vector<VkQueueFamilyProper
     }
 }
 
-Device::~Device() NOEXCEPT {
+Device::~Device() noexcept {
     if (!initialized()) return;
 
     vk::DestroyDevice(handle(), NULL);
@@ -261,9 +263,7 @@ void Device::init(std::vector<const char *> &extensions, VkPhysicalDeviceFeature
 
     enabled_extensions_ = extensions;
 
-    VkDeviceCreateInfo dev_info = {};
-    dev_info.sType = VK_STRUCTURE_TYPE_DEVICE_CREATE_INFO;
-    dev_info.pNext = create_device_pnext;
+    VkDeviceCreateInfo dev_info = LvlInitStruct<VkDeviceCreateInfo>(create_device_pnext);
     dev_info.queueCreateInfoCount = create_queue_infos.size();
     dev_info.pQueueCreateInfos = create_queue_infos.data();
     dev_info.enabledLayerCount = 0;
@@ -275,7 +275,7 @@ void Device::init(std::vector<const char *> &extensions, VkPhysicalDeviceFeature
     // Let VkPhysicalDeviceFeatures2 take priority over VkPhysicalDeviceFeatures,
     // since it supports extensions
 
-    if (!(lvl_find_in_chain<VkPhysicalDeviceFeatures2>(dev_info.pNext))) {
+    if (!(LvlFindInChain<VkPhysicalDeviceFeatures2>(dev_info.pNext))) {
         if (features) {
             dev_info.pEnabledFeatures = features;
         } else {
@@ -293,11 +293,11 @@ void Device::init(const VkDeviceCreateInfo &info) {
 
     if (EXPECT(vk::CreateDevice(phy_.handle(), &info, NULL, &dev) == VK_SUCCESS)) Handle::init(dev);
 
-    init_queues();
+    init_queues(info);
     init_formats();
 }
 
-void Device::init_queues() {
+void Device::init_queues(const VkDeviceCreateInfo &info) {
     uint32_t queue_node_count;
     vk::GetPhysicalDeviceQueueFamilyProperties(phy_.handle(), &queue_node_count, NULL);
     EXPECT(queue_node_count >= 1);
@@ -306,35 +306,37 @@ void Device::init_queues() {
     vk::GetPhysicalDeviceQueueFamilyProperties(phy_.handle(), &queue_node_count, queue_props.data());
 
     queue_families_.resize(queue_node_count);
-    for (uint32_t queue_family_i = 0; queue_family_i < queue_node_count; ++queue_family_i) {
-        const auto &queue_prop = queue_props[queue_family_i];
+    for (uint32_t i = 0; i < info.queueCreateInfoCount; i++) {
+        const auto &queue_create_info = info.pQueueCreateInfos[i];
+        auto queue_family_i = queue_create_info.queueFamilyIndex;
+        const auto &queue_family_prop = queue_props[queue_family_i];
 
         QueueFamilyQueues &queue_storage = queue_families_[queue_family_i];
-        queue_storage.reserve(queue_prop.queueCount);
-        for (uint32_t queue_i = 0; queue_i < queue_prop.queueCount; ++queue_i) {
+        queue_storage.reserve(queue_create_info.queueCount);
+        for (uint32_t queue_i = 0; queue_i < queue_create_info.queueCount; ++queue_i) {
             // TODO: Need to add support for separate MEMMGR and work queues,
             // including synchronization
-            VkQueue queue;
+            VkQueue queue = VK_NULL_HANDLE;
             vk::GetDeviceQueue(handle(), queue_family_i, queue_i, &queue);
 
             // Store single copy of the queue object that will self destruct
             queue_storage.emplace_back(new Queue(queue, queue_family_i));
 
-            if (queue_prop.queueFlags & VK_QUEUE_GRAPHICS_BIT) {
+            if (queue_family_prop.queueFlags & VK_QUEUE_GRAPHICS_BIT) {
                 queues_[GRAPHICS].push_back(queue_storage.back().get());
             }
 
-            if (queue_prop.queueFlags & VK_QUEUE_COMPUTE_BIT) {
+            if (queue_family_prop.queueFlags & VK_QUEUE_COMPUTE_BIT) {
                 queues_[COMPUTE].push_back(queue_storage.back().get());
             }
 
-            if (queue_prop.queueFlags & VK_QUEUE_TRANSFER_BIT) {
+            if (queue_family_prop.queueFlags & VK_QUEUE_TRANSFER_BIT) {
                 queues_[DMA].push_back(queue_storage.back().get());
             }
         }
     }
 
-    EXPECT(!queues_[GRAPHICS].empty() || !queues_[COMPUTE].empty());
+    EXPECT(!queues_[GRAPHICS].empty() || !queues_[COMPUTE].empty() || !queues_[DMA].empty());
 }
 
 const Device::QueueFamilyQueues &Device::queue_family_queues(uint32_t queue_family) const {
@@ -391,16 +393,14 @@ void Device::update_descriptor_sets(const std::vector<VkWriteDescriptorSet> &wri
 
 VkResult Queue::submit(const std::vector<const CommandBuffer *> &cmds, const Fence &fence, bool expect_success) {
     const std::vector<VkCommandBuffer> cmd_handles = MakeVkHandles<VkCommandBuffer>(cmds);
-    VkSubmitInfo submit_info;
-    submit_info.sType = VK_STRUCTURE_TYPE_SUBMIT_INFO;
-    submit_info.pNext = NULL;
+    VkSubmitInfo submit_info = LvlInitStruct<VkSubmitInfo>();
     submit_info.waitSemaphoreCount = 0;
-    submit_info.pWaitSemaphores = NULL;
-    submit_info.pWaitDstStageMask = NULL;
-    submit_info.commandBufferCount = (uint32_t)cmd_handles.size();
+    submit_info.pWaitSemaphores = nullptr;
+    submit_info.pWaitDstStageMask = nullptr;
+    submit_info.commandBufferCount = static_cast<uint32_t>(cmd_handles.size());
     submit_info.pCommandBuffers = cmd_handles.data();
     submit_info.signalSemaphoreCount = 0;
-    submit_info.pSignalSemaphores = NULL;
+    submit_info.pSignalSemaphores = nullptr;
 
     VkResult result = vk::QueueSubmit(handle(), 1, &submit_info, fence.handle());
     if (expect_success) EXPECT(result == VK_SUCCESS);
@@ -416,13 +416,41 @@ VkResult Queue::submit(const CommandBuffer &cmd, bool expect_success) {
     return submit(cmd, fence);
 }
 
+VkResult Queue::submit2(const std::vector<const CommandBuffer *> &cmds, const Fence &fence, bool expect_success) {
+    std::vector<VkCommandBufferSubmitInfo> cmd_submit_infos;
+    for (size_t i = 0; i < cmds.size(); i++) {
+        VkCommandBufferSubmitInfo cmd_submit_info = LvlInitStruct<VkCommandBufferSubmitInfo>();
+        cmd_submit_info.deviceMask = 0;
+        cmd_submit_info.commandBuffer = cmds[i]->handle();
+        cmd_submit_infos.push_back(cmd_submit_info);
+    }
+
+    VkSubmitInfo2 submit_info = LvlInitStruct<VkSubmitInfo2>();
+    submit_info.flags = 0;
+    submit_info.waitSemaphoreInfoCount = 0;
+    submit_info.pWaitSemaphoreInfos = nullptr;
+    submit_info.signalSemaphoreInfoCount = 0;
+    submit_info.pSignalSemaphoreInfos = nullptr;
+    submit_info.commandBufferInfoCount = static_cast<uint32_t>(cmd_submit_infos.size());
+    submit_info.pCommandBufferInfos = cmd_submit_infos.data();
+
+    // requires synchronization2 to be enabled
+    VkResult result = vk::QueueSubmit2(handle(), 1, &submit_info, fence.handle());
+    if (expect_success) EXPECT(result == VK_SUCCESS);
+    return result;
+}
+
+VkResult Queue::submit2(const CommandBuffer &cmd, const Fence &fence, bool expect_success) {
+    return submit2(std::vector<const CommandBuffer *>(1, &cmd), fence, expect_success);
+}
+
 VkResult Queue::wait() {
     VkResult result = vk::QueueWaitIdle(handle());
     EXPECT(result == VK_SUCCESS);
     return result;
 }
 
-DeviceMemory::~DeviceMemory() NOEXCEPT {
+DeviceMemory::~DeviceMemory() noexcept {
     if (initialized()) vk::FreeMemory(device(), handle(), NULL);
 }
 
@@ -470,18 +498,123 @@ VkResult Fence::wait(uint64_t timeout) const {
     return vk::WaitForFences(device(), 1, &fence, VK_TRUE, timeout);
 }
 
+VkResult Fence::reset() {
+    VkFence fence = handle();
+    return vk::ResetFences(device(), 1, &fence);
+}
+
+VkResult Fence::export_handle(Fence::ExternalHandle &ext_handle, VkExternalFenceHandleTypeFlagBits handle_type) {
+#ifdef _WIN32
+    auto ghi = LvlInitStruct<VkFenceGetWin32HandleInfoKHR>();
+    ghi.fence = handle();
+    ghi.handleType = handle_type;
+    auto vkGetFenceWin32HandleKHR = (PFN_vkGetFenceWin32HandleKHR)vk::GetDeviceProcAddr(device(), "vkGetFenceWin32HandleKHR");
+    return vkGetFenceWin32HandleKHR(device(), &ghi, &ext_handle);
+#else
+    auto gfi = LvlInitStruct<VkFenceGetFdInfoKHR>();
+    gfi.fence = handle();
+    gfi.handleType = handle_type;
+    auto vkGetFenceFdKHR = (PFN_vkGetFenceFdKHR)vk::GetDeviceProcAddr(device(), "vkGetFenceFdKHR");
+    return vkGetFenceFdKHR(device(), &gfi, &ext_handle);
+#endif
+}
+
+VkResult Fence::import_handle(Fence::ExternalHandle ext_handle, VkExternalFenceHandleTypeFlagBits handle_type,
+                              VkFenceImportFlags flags) {
+#ifdef _WIN32
+    auto ifi = LvlInitStruct<VkImportFenceWin32HandleInfoKHR>();
+    ifi.fence = handle();
+    ifi.handleType = handle_type;
+    ifi.handle = ext_handle;
+    ifi.flags = flags;
+    auto vkImportFenceWin32HandleKHR =
+        reinterpret_cast<PFN_vkImportFenceWin32HandleKHR>(vk::GetDeviceProcAddr(device(), "vkImportFenceWin32HandleKHR"));
+    return vkImportFenceWin32HandleKHR(device(), &ifi);
+#else
+    auto ifi = LvlInitStruct<VkImportFenceFdInfoKHR>();
+    ifi.fence = handle();
+    ifi.handleType = handle_type;
+    ifi.fd = ext_handle;
+    ifi.flags = flags;
+    auto vkImportFenceFdKHR = reinterpret_cast<PFN_vkImportFenceFdKHR>(vk::GetDeviceProcAddr(device(), "vkImportFenceFdKHR"));
+    return vkImportFenceFdKHR(device(), &ifi);
+#endif
+}
+
 NON_DISPATCHABLE_HANDLE_DTOR(Semaphore, vk::DestroySemaphore)
 
 void Semaphore::init(const Device &dev, const VkSemaphoreCreateInfo &info) {
     NON_DISPATCHABLE_HANDLE_INIT(vk::CreateSemaphore, dev, &info);
 }
 
+VkResult Semaphore::export_handle(Semaphore::ExternalHandle &ext_handle, VkExternalSemaphoreHandleTypeFlagBits handle_type) {
+#ifdef _WIN32
+    ext_handle = nullptr;
+    auto ghi = LvlInitStruct<VkSemaphoreGetWin32HandleInfoKHR>();
+    ghi.semaphore = handle();
+    ghi.handleType = handle_type;
+    auto vkGetSemaphoreWin32HandleKHR =
+        reinterpret_cast<PFN_vkGetSemaphoreWin32HandleKHR>(vk::GetDeviceProcAddr(device(), "vkGetSemaphoreWin32HandleKHR"));
+    return vkGetSemaphoreWin32HandleKHR(device(), &ghi, &ext_handle);
+#else
+    ext_handle = -1;
+    auto ghi = LvlInitStruct<VkSemaphoreGetFdInfoKHR>();
+    ghi.semaphore = handle();
+    ghi.handleType = handle_type;
+    auto vkGetSemaphoreFdKHR = reinterpret_cast<PFN_vkGetSemaphoreFdKHR>(vk::GetDeviceProcAddr(device(), "vkGetSemaphoreFdKHR"));
+    return vkGetSemaphoreFdKHR(device(), &ghi, &ext_handle);
+#endif
+}
+
+VkResult Semaphore::import_handle(Semaphore::ExternalHandle ext_handle, VkExternalSemaphoreHandleTypeFlagBits handle_type,
+                                  VkSemaphoreImportFlags flags) {
+#ifdef _WIN32
+    auto ihi = LvlInitStruct<VkImportSemaphoreWin32HandleInfoKHR>();
+    ihi.semaphore = handle();
+    ihi.handleType = handle_type;
+    ihi.handle = ext_handle;
+    ihi.flags = flags;
+
+    auto vkImportSemaphoreWin32HandleKHR =
+        reinterpret_cast<PFN_vkImportSemaphoreWin32HandleKHR>(vk::GetDeviceProcAddr(device(), "vkImportSemaphoreWin32HandleKHR"));
+    return vkImportSemaphoreWin32HandleKHR(device(), &ihi);
+#else
+    // Import opaque handle exported above
+    auto ihi = LvlInitStruct<VkImportSemaphoreFdInfoKHR>();
+    ihi.semaphore = handle();
+    ihi.handleType = handle_type;
+    ihi.fd = ext_handle;
+    ihi.flags = flags;
+
+    auto vkImportSemaphoreFdKHR =
+        reinterpret_cast<PFN_vkImportSemaphoreFdKHR>(vk::GetDeviceProcAddr(device(), "vkImportSemaphoreFdKHR"));
+    return vkImportSemaphoreFdKHR(device(), &ihi);
+#endif
+}
+
 NON_DISPATCHABLE_HANDLE_DTOR(Event, vk::DestroyEvent)
 
 void Event::init(const Device &dev, const VkEventCreateInfo &info) { NON_DISPATCHABLE_HANDLE_INIT(vk::CreateEvent, dev, &info); }
 
 void Event::set() { EXPECT(vk::SetEvent(device(), handle()) == VK_SUCCESS); }
 
+void Event::cmd_set(const CommandBuffer &cmd, VkPipelineStageFlags stage_mask) {
+    vk::CmdSetEvent(cmd.handle(), handle(), stage_mask);
+}
+
+void Event::cmd_reset(const CommandBuffer &cmd, VkPipelineStageFlags stage_mask) {
+    vk::CmdResetEvent(cmd.handle(), handle(), stage_mask);
+}
+
+void Event::cmd_wait(const CommandBuffer &cmd, VkPipelineStageFlags src_stage_mask, VkPipelineStageFlags dst_stage_mask,
+                     const std::vector<VkMemoryBarrier> &memory_barriers, const std::vector<VkBufferMemoryBarrier> &buffer_barriers,
+                     const std::vector<VkImageMemoryBarrier> &image_barriers) {
+    VkEvent event_handle = handle();
+    vk::CmdWaitEvents(cmd.handle(), 1, &event_handle, src_stage_mask, dst_stage_mask, static_cast<uint32_t>(memory_barriers.size()),
+                      memory_barriers.data(), static_cast<uint32_t>(buffer_barriers.size()), buffer_barriers.data(),
+                      static_cast<uint32_t>(image_barriers.size()), image_barriers.data());
+}
+
 void Event::reset() { EXPECT(vk::ResetEvent(device(), handle()) == VK_SUCCESS); }
 
 NON_DISPATCHABLE_HANDLE_DTOR(QueryPool, vk::DestroyQueryPool)
@@ -499,10 +632,13 @@ VkResult QueryPool::results(uint32_t first, uint32_t count, size_t size, void *d
 
 NON_DISPATCHABLE_HANDLE_DTOR(Buffer, vk::DestroyBuffer)
 
-void Buffer::init(const Device &dev, const VkBufferCreateInfo &info, VkMemoryPropertyFlags mem_props) {
+void Buffer::init(const Device &dev, const VkBufferCreateInfo &info, VkMemoryPropertyFlags mem_props, void *alloc_info_pnext) {
     init_no_mem(dev, info);
 
-    internal_mem_.init(dev, DeviceMemory::get_resource_alloc_info(dev, memory_requirements(), mem_props));
+    auto alloc_info = DeviceMemory::get_resource_alloc_info(dev, memory_requirements(), mem_props);
+    alloc_info.pNext = alloc_info_pnext;
+    internal_mem_.init(dev, alloc_info);
+
     bind_memory(internal_mem_, 0);
 }
 
@@ -523,6 +659,13 @@ void Buffer::bind_memory(const DeviceMemory &mem, VkDeviceSize mem_offset) {
     EXPECT(vk::BindBufferMemory(device(), handle(), mem.handle(), mem_offset) == VK_SUCCESS);
 }
 
+void Buffer::bind_memory(const Device &dev, VkMemoryPropertyFlags mem_props, VkDeviceSize mem_offset) {
+    if (!internal_mem_.initialized()) {
+        internal_mem_.init(dev, DeviceMemory::get_resource_alloc_info(dev, memory_requirements(), mem_props));
+    }
+    bind_memory(internal_mem_, mem_offset);
+}
+
 NON_DISPATCHABLE_HANDLE_DTOR(BufferView, vk::DestroyBufferView)
 
 void BufferView::init(const Device &dev, const VkBufferViewCreateInfo &info) {
@@ -594,6 +737,20 @@ bool Image::transparent() const {
             !(create_info_.usage & (VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT | VK_IMAGE_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT)));
 }
 
+VkImageAspectFlags Image::aspect_mask(VkFormat format) {
+    VkImageAspectFlags image_aspect;
+    if (FormatIsDepthAndStencil(format)) {
+        image_aspect = VK_IMAGE_ASPECT_STENCIL_BIT | VK_IMAGE_ASPECT_DEPTH_BIT;
+    } else if (FormatIsDepthOnly(format)) {
+        image_aspect = VK_IMAGE_ASPECT_DEPTH_BIT;
+    } else if (FormatIsStencilOnly(format)) {
+        image_aspect = VK_IMAGE_ASPECT_STENCIL_BIT;
+    } else {  // color
+        image_aspect = VK_IMAGE_ASPECT_COLOR_BIT;
+    }
+    return image_aspect;
+}
+
 NON_DISPATCHABLE_HANDLE_DTOR(ImageView, vk::DestroyImageView)
 
 void ImageView::init(const Device &dev, const VkImageViewCreateInfo &info) {
@@ -610,17 +767,24 @@ AccelerationStructure::~AccelerationStructure() {
     }
 }
 
+AccelerationStructureKHR::~AccelerationStructureKHR() {
+    if (initialized()) {
+        PFN_vkDestroyAccelerationStructureKHR vkDestroyAccelerationStructureKHR =
+            (PFN_vkDestroyAccelerationStructureKHR)vk::GetDeviceProcAddr(device(), "vkDestroyAccelerationStructureKHR");
+        assert(vkDestroyAccelerationStructureKHR != nullptr);
+        vkDestroyAccelerationStructureKHR(device(), handle(), nullptr);
+    }
+}
 VkMemoryRequirements2 AccelerationStructure::memory_requirements() const {
     PFN_vkGetAccelerationStructureMemoryRequirementsNV vkGetAccelerationStructureMemoryRequirementsNV =
         (PFN_vkGetAccelerationStructureMemoryRequirementsNV)vk::GetDeviceProcAddr(device(),
                                                                                   "vkGetAccelerationStructureMemoryRequirementsNV");
     assert(vkGetAccelerationStructureMemoryRequirementsNV != nullptr);
-
-    VkAccelerationStructureMemoryRequirementsInfoNV memoryRequirementsInfo = {};
-    memoryRequirementsInfo.sType = VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_MEMORY_REQUIREMENTS_INFO_NV;
+    VkMemoryRequirements2 memoryRequirements = {};
+    VkAccelerationStructureMemoryRequirementsInfoNV memoryRequirementsInfo =
+        LvlInitStruct<VkAccelerationStructureMemoryRequirementsInfoNV>();
     memoryRequirementsInfo.type = VK_ACCELERATION_STRUCTURE_MEMORY_REQUIREMENTS_TYPE_OBJECT_NV;
     memoryRequirementsInfo.accelerationStructure = handle();
-    VkMemoryRequirements2 memoryRequirements = {};
     vkGetAccelerationStructureMemoryRequirementsNV(device(), &memoryRequirementsInfo, &memoryRequirements);
     return memoryRequirements;
 }
@@ -631,8 +795,8 @@ VkMemoryRequirements2 AccelerationStructure::build_scratch_memory_requirements()
                                                                                   "vkGetAccelerationStructureMemoryRequirementsNV");
     assert(vkGetAccelerationStructureMemoryRequirementsNV != nullptr);
 
-    VkAccelerationStructureMemoryRequirementsInfoNV memoryRequirementsInfo = {};
-    memoryRequirementsInfo.sType = VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_MEMORY_REQUIREMENTS_INFO_NV;
+    VkAccelerationStructureMemoryRequirementsInfoNV memoryRequirementsInfo =
+        LvlInitStruct<VkAccelerationStructureMemoryRequirementsInfoNV>();
     memoryRequirementsInfo.type = VK_ACCELERATION_STRUCTURE_MEMORY_REQUIREMENTS_TYPE_BUILD_SCRATCH_NV;
     memoryRequirementsInfo.accelerationStructure = handle();
 
@@ -658,8 +822,7 @@ void AccelerationStructure::init(const Device &dev, const VkAccelerationStructur
             (PFN_vkBindAccelerationStructureMemoryNV)vk::GetDeviceProcAddr(dev.handle(), "vkBindAccelerationStructureMemoryNV");
         assert(vkBindAccelerationStructureMemoryNV != nullptr);
 
-        VkBindAccelerationStructureMemoryInfoNV bind_info = {};
-        bind_info.sType = VK_STRUCTURE_TYPE_BIND_ACCELERATION_STRUCTURE_MEMORY_INFO_NV;
+        VkBindAccelerationStructureMemoryInfoNV bind_info = LvlInitStruct<VkBindAccelerationStructureMemoryInfoNV>();
         bind_info.accelerationStructure = handle();
         bind_info.memory = memory_.handle();
         EXPECT(vkBindAccelerationStructureMemoryNV(dev.handle(), 1, &bind_info) == VK_SUCCESS);
@@ -670,10 +833,9 @@ void AccelerationStructure::init(const Device &dev, const VkAccelerationStructur
         EXPECT(vkGetAccelerationStructureHandleNV(dev.handle(), handle(), sizeof(uint64_t), &opaque_handle_) == VK_SUCCESS);
     }
 }
-
-void AccelerationStructure::create_scratch_buffer(const Device &dev, Buffer *buffer, VkBufferCreateInfo *pCreateInfo) {
+void AccelerationStructure::create_scratch_buffer(const Device &dev, Buffer *buffer, VkBufferCreateInfo *pCreateInfo,
+                                                  bool buffer_device_address) {
     VkMemoryRequirements scratch_buffer_memory_requirements = build_scratch_memory_requirements().memoryRequirements;
-
     VkBufferCreateInfo create_info = {};
     create_info.size = scratch_buffer_memory_requirements.size;
     if (pCreateInfo) {
@@ -682,8 +844,44 @@ void AccelerationStructure::create_scratch_buffer(const Device &dev, Buffer *buf
     } else {
         create_info.sType = VK_STRUCTURE_TYPE_BUFFER_CREATE_INFO;
         create_info.usage = VK_BUFFER_USAGE_RAY_TRACING_BIT_NV;
+        if (buffer_device_address) create_info.usage |= VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT;
+    }
+    if (buffer_device_address) {
+        auto alloc_flags = LvlInitStruct<VkMemoryAllocateFlagsInfo>();
+        alloc_flags.flags = VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_BIT_KHR;
+        buffer->init(dev, create_info, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT, &alloc_flags);
+    } else {
+        buffer->init(dev, create_info, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
+    }
+}
+
+void AccelerationStructureKHR::init(const Device &dev, const VkAccelerationStructureCreateInfoKHR &info, bool init_memory) {
+    PFN_vkCreateAccelerationStructureKHR vkCreateAccelerationStructureKHR =
+        (PFN_vkCreateAccelerationStructureKHR)vk::GetDeviceProcAddr(dev.handle(), "vkCreateAccelerationStructureKHR");
+    assert(vkCreateAccelerationStructureKHR != nullptr);
+    NON_DISPATCHABLE_HANDLE_INIT(vkCreateAccelerationStructureKHR, dev, &info);
+    info_ = info;
+}
+void AccelerationStructureKHR::create_scratch_buffer(const Device &dev, Buffer *buffer, VkBufferCreateInfo *pCreateInfo,
+                                                     bool buffer_device_address) {
+    VkBufferCreateInfo create_info = {};
+    create_info.size = 0;
+    if (pCreateInfo) {
+        create_info.sType = pCreateInfo->sType;
+        create_info.usage = pCreateInfo->usage;
+        create_info.size = pCreateInfo->size;
+    } else {
+        create_info.sType = VK_STRUCTURE_TYPE_BUFFER_CREATE_INFO;
+        create_info.usage = VK_BUFFER_USAGE_RAY_TRACING_BIT_NV;
+        if (buffer_device_address) create_info.usage |= VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT;
+    }
+    if (buffer_device_address) {
+        auto alloc_flags = LvlInitStruct<VkMemoryAllocateFlagsInfo>();
+        alloc_flags.flags = VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_BIT_KHR;
+        buffer->init(dev, create_info, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT, &alloc_flags);
+    } else {
+        buffer->init(dev, create_info, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
     }
-    buffer->init(dev, create_info, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
 }
 
 NON_DISPATCHABLE_HANDLE_DTOR(ShaderModule, vk::DestroyShaderModule)
@@ -705,9 +903,7 @@ NON_DISPATCHABLE_HANDLE_DTOR(Pipeline, vk::DestroyPipeline)
 
 void Pipeline::init(const Device &dev, const VkGraphicsPipelineCreateInfo &info) {
     VkPipelineCache cache;
-    VkPipelineCacheCreateInfo ci;
-    memset((void *)&ci, 0, sizeof(VkPipelineCacheCreateInfo));
-    ci.sType = VK_STRUCTURE_TYPE_PIPELINE_CACHE_CREATE_INFO;
+    VkPipelineCacheCreateInfo ci = LvlInitStruct<VkPipelineCacheCreateInfo>();
     VkResult err = vk::CreatePipelineCache(dev.handle(), &ci, NULL, &cache);
     if (err == VK_SUCCESS) {
         NON_DISPATCHABLE_HANDLE_INIT(vk::CreateGraphicsPipelines, dev, cache, 1, &info);
@@ -718,9 +914,7 @@ void Pipeline::init(const Device &dev, const VkGraphicsPipelineCreateInfo &info)
 VkResult Pipeline::init_try(const Device &dev, const VkGraphicsPipelineCreateInfo &info) {
     VkPipeline pipe;
     VkPipelineCache cache;
-    VkPipelineCacheCreateInfo ci;
-    memset((void *)&ci, 0, sizeof(VkPipelineCacheCreateInfo));
-    ci.sType = VK_STRUCTURE_TYPE_PIPELINE_CACHE_CREATE_INFO;
+    VkPipelineCacheCreateInfo ci = LvlInitStruct<VkPipelineCacheCreateInfo>();
     VkResult err = vk::CreatePipelineCache(dev.handle(), &ci, NULL, &cache);
     EXPECT(err == VK_SUCCESS);
     if (err == VK_SUCCESS) {
@@ -736,9 +930,7 @@ VkResult Pipeline::init_try(const Device &dev, const VkGraphicsPipelineCreateInf
 
 void Pipeline::init(const Device &dev, const VkComputePipelineCreateInfo &info) {
     VkPipelineCache cache;
-    VkPipelineCacheCreateInfo ci;
-    memset((void *)&ci, 0, sizeof(VkPipelineCacheCreateInfo));
-    ci.sType = VK_STRUCTURE_TYPE_PIPELINE_CACHE_CREATE_INFO;
+    VkPipelineCacheCreateInfo ci = LvlInitStruct<VkPipelineCacheCreateInfo>();
     VkResult err = vk::CreatePipelineCache(dev.handle(), &ci, NULL, &cache);
     if (err == VK_SUCCESS) {
         NON_DISPATCHABLE_HANDLE_INIT(vk::CreateComputePipelines, dev, cache, 1, &info);
@@ -785,8 +977,7 @@ std::vector<DescriptorSet *> DescriptorPool::alloc_sets(const Device &dev,
     std::vector<VkDescriptorSet> set_handles;
     set_handles.resize(layout_handles.size());
 
-    VkDescriptorSetAllocateInfo alloc_info = {};
-    alloc_info.sType = VK_STRUCTURE_TYPE_DESCRIPTOR_SET_ALLOCATE_INFO;
+    VkDescriptorSetAllocateInfo alloc_info = LvlInitStruct<VkDescriptorSetAllocateInfo>();
     alloc_info.descriptorSetCount = layout_handles.size();
     alloc_info.descriptorPool = handle();
     alloc_info.pSetLayouts = layout_handles.data();
@@ -811,7 +1002,7 @@ DescriptorSet *DescriptorPool::alloc_sets(const Device &dev, const DescriptorSet
     return (set.empty()) ? NULL : set[0];
 }
 
-DescriptorSet::~DescriptorSet() NOEXCEPT {
+DescriptorSet::~DescriptorSet() noexcept {
     if (initialized()) {
         // Only call vk::Free* on sets allocated from pool with usage *_DYNAMIC
         if (containing_pool_->getDynamicUsage()) {
@@ -827,7 +1018,7 @@ void CommandPool::init(const Device &dev, const VkCommandPoolCreateInfo &info) {
     NON_DISPATCHABLE_HANDLE_INIT(vk::CreateCommandPool, dev, &info);
 }
 
-CommandBuffer::~CommandBuffer() NOEXCEPT {
+CommandBuffer::~CommandBuffer() noexcept {
     if (initialized()) {
         VkCommandBuffer cmds[] = {handle()};
         vk::FreeCommandBuffers(dev_handle_, cmd_pool_, 1, cmds);
@@ -850,13 +1041,10 @@ void CommandBuffer::init(const Device &dev, const VkCommandBufferAllocateInfo &i
 void CommandBuffer::begin(const VkCommandBufferBeginInfo *info) { EXPECT(vk::BeginCommandBuffer(handle(), info) == VK_SUCCESS); }
 
 void CommandBuffer::begin() {
-    VkCommandBufferBeginInfo info = {};
-    VkCommandBufferInheritanceInfo hinfo = {};
+    VkCommandBufferBeginInfo info = LvlInitStruct<VkCommandBufferBeginInfo>();
+    VkCommandBufferInheritanceInfo hinfo = LvlInitStruct<VkCommandBufferInheritanceInfo>();
     info.flags = VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT;
-    info.sType = VK_STRUCTURE_TYPE_COMMAND_BUFFER_BEGIN_INFO;
     info.pInheritanceInfo = &hinfo;
-    hinfo.sType = VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_INFO;
-    hinfo.pNext = NULL;
     hinfo.renderPass = VK_NULL_HANDLE;
     hinfo.subpass = 0;
     hinfo.framebuffer = VK_NULL_HANDLE;
@@ -871,4 +1059,73 @@ void CommandBuffer::end() { EXPECT(vk::EndCommandBuffer(handle()) == VK_SUCCESS)
 
 void CommandBuffer::reset(VkCommandBufferResetFlags flags) { EXPECT(vk::ResetCommandBuffer(handle(), flags) == VK_SUCCESS); }
 
+void RenderPass::init(const Device &dev, const VkRenderPassCreateInfo &info) {
+    NON_DISPATCHABLE_HANDLE_INIT(vk::CreateRenderPass, dev, &info);
+}
+
+void RenderPass::init(const Device &dev, const VkRenderPassCreateInfo2 &info, bool khr) {
+    if (!khr) {
+        NON_DISPATCHABLE_HANDLE_INIT(vk::CreateRenderPass2, dev, &info);
+    } else {
+        auto vkCreateRenderPass2KHR =
+            reinterpret_cast<PFN_vkCreateRenderPass2KHR>(vk::GetDeviceProcAddr(dev.handle(), "vkCreateRenderPass2KHR"));
+        ASSERT_NE(vkCreateRenderPass2KHR, nullptr);
+        NON_DISPATCHABLE_HANDLE_INIT(vkCreateRenderPass2KHR, dev, &info);
+    }
+}
+
+NON_DISPATCHABLE_HANDLE_DTOR(RenderPass, vk::DestroyRenderPass)
+
+void Framebuffer::init(const Device &dev, const VkFramebufferCreateInfo &info) {
+    NON_DISPATCHABLE_HANDLE_INIT(vk::CreateFramebuffer, dev, &info);
+}
+
+NON_DISPATCHABLE_HANDLE_DTOR(Framebuffer, vk::DestroyFramebuffer)
+
+void SamplerYcbcrConversion::init(const Device &dev, const VkSamplerYcbcrConversionCreateInfo &info, bool khr) {
+    if (!khr) {
+        NON_DISPATCHABLE_HANDLE_INIT(vk::CreateSamplerYcbcrConversion, dev, &info);
+    } else {
+        auto vkCreateSamplerYcbcrConversionKHR = reinterpret_cast<PFN_vkCreateSamplerYcbcrConversionKHR>(
+            vk::GetDeviceProcAddr(dev.handle(), "vkCreateSamplerYcbcrConversionKHR"));
+        ASSERT_NE(vkCreateSamplerYcbcrConversionKHR, nullptr);
+        NON_DISPATCHABLE_HANDLE_INIT(vkCreateSamplerYcbcrConversionKHR, dev, &info);
+    }
+}
+
+VkSamplerYcbcrConversionInfo SamplerYcbcrConversion::ConversionInfo() {
+    VkSamplerYcbcrConversionInfo ycbcr_info = LvlInitStruct<VkSamplerYcbcrConversionInfo>();
+    ycbcr_info.conversion = handle();
+    return ycbcr_info;
+}
+
+// static
+VkSamplerYcbcrConversionCreateInfo SamplerYcbcrConversion::DefaultConversionInfo(VkFormat format) {
+    auto ycbcr_create_info = LvlInitStruct<VkSamplerYcbcrConversionCreateInfo>();
+    ycbcr_create_info.format = format;
+    ycbcr_create_info.ycbcrModel = VK_SAMPLER_YCBCR_MODEL_CONVERSION_RGB_IDENTITY;
+    ycbcr_create_info.ycbcrRange = VK_SAMPLER_YCBCR_RANGE_ITU_FULL;
+    ycbcr_create_info.components = {VK_COMPONENT_SWIZZLE_IDENTITY, VK_COMPONENT_SWIZZLE_IDENTITY, VK_COMPONENT_SWIZZLE_IDENTITY,
+                                    VK_COMPONENT_SWIZZLE_IDENTITY};
+    ycbcr_create_info.xChromaOffset = VK_CHROMA_LOCATION_COSITED_EVEN;
+    ycbcr_create_info.yChromaOffset = VK_CHROMA_LOCATION_COSITED_EVEN;
+    ycbcr_create_info.chromaFilter = VK_FILTER_NEAREST;
+    ycbcr_create_info.forceExplicitReconstruction = false;
+    return ycbcr_create_info;
+}
+
+SamplerYcbcrConversion::~SamplerYcbcrConversion() noexcept {
+    if (initialized()) {
+        if (!khr_) {
+            vk::DestroySamplerYcbcrConversion(device(), handle(), nullptr);
+        } else {
+            auto vkDestroySamplerYcbcrConversionKHR = reinterpret_cast<PFN_vkDestroySamplerYcbcrConversionKHR>(
+                vk::GetDeviceProcAddr(device(), "vkDestroySamplerYcbcrConversionKHR"));
+            assert(vkDestroySamplerYcbcrConversionKHR != nullptr);
+            vkDestroySamplerYcbcrConversionKHR(device(), handle(), nullptr);
+        }
+        handle_ = VK_NULL_HANDLE;
+    }
+}
+
 }  // namespace vk_testing
